@@ -8,18 +8,20 @@ use POSIX ();
 
 use Types::Standard qw( ArrayRef Bool Dict Enum HashRef InstanceOf Optional Slurpy );
 use Type::Params    qw( signature signature_for );
-use Ref::Util       qw[ is_plain_hashref is_blessed_ref ];
+use Ref::Util       qw( is_plain_hashref is_blessed_ref );
+use List::Util      qw( uniqnum );
 
 use CXC::Number::Grid::Types -types;
 use CXC::Number::Grid::Failure qw( parameter_interface parameter_constraint internal );
 
 use CXC::Number::Grid::Tree;
+use constant Tree => 'CXC::Number::Grid::Tree';
 
 use Safe::Isa;
 
 use Moo;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use constant GridObject   => InstanceOf [ ( __PACKAGE__ ) ];
 use constant IncludeArray => ArrayRef [ Enum [ 0, 1 ] ];
@@ -35,6 +37,7 @@ use MooX::StrictConstructor;
 use overload
   '!'      => \&_overload_not,
   '|'      => \&_overload_or,
+  '&'      => \&_overload_and,
   fallback => 1,
   bool     => sub { 1 };
 
@@ -524,7 +527,7 @@ sub overlay_n {
     $opt->{snap_to}   //= 'underlay';
     $opt->{snap_dist} //= Math::BigFloat->bzero;
 
-    my $tr = CXC::Number::Grid::Tree->new;
+    my $tr = Tree->new;
 
     my $gi = 0;
     for my $grid ( $grids->@* ) {
@@ -865,10 +868,6 @@ sub not ( $self ) {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
     return __PACKAGE__->new( $hash );
 }
 
-
-
-
-
 sub _overload_not ( $self, $, $ ) {
     return $self->not;
 }
@@ -928,16 +927,21 @@ sub or ( $self, @args ) {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
 
 
 
+
+
+
 
 
 
 
 signature_for or_n => ( positional => [ Slurpy [ ArrayRef [GridObject] ] ], );
 sub or_n ( $grids ) {
-    my $tr = CXC::Number::Grid::Tree->new;
+    my $tr = Tree->new;
 
     my $gi = 0;
 
+    # the code for and_n is easier to understand; rewrite this to use
+    # that algorithm.
     for my $grid ( $grids->@* ) {
         ++$gi;
         my $edges   = $grid->_raw_edges;
@@ -965,6 +969,99 @@ sub or_n ( $grids ) {
 
 
 
+# need extra args if bitwise feature is on.
+sub _overload_and ( $self, $other, $ =, $ =, $ = ) {
+    $other->$_isa( __PACKAGE__ )
+      or die( "can only perform the | operation between two ${ \__PACKAGE__ } objects " );
+    return $self->and( $other );
+}
+
+
+
+
+
+
+
+
+
+
+sub and ( $self, @args ) {    ## no critic (Subroutines::ProhibitBuiltinHomonyms)
+    return and_n( $self, @args );
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+signature_for and_n => ( positional => [ Slurpy [ ArrayRef [GridObject] ] ], );
+sub and_n ( $grids ) {
+
+
+    # need to get all of the grid's bins on the same grid.
+
+    # create a sorted list of bin edges from all of the grids, then,
+    # ask each grid what it's include value is for each bin and
+
+    my @edge    = uniqnum sort { $a <=> $b } map { $_->_raw_edges->@* } $grids->@*;
+    my @include = ( 1 ) x ( @edge - 1 );
+    my $nbins   = @include;
+
+    for my $grid ( $grids->@* ) {
+        my $tree = Tree->from_grid( $grid );
+
+        # we don't care about the last edge, as anything beyond that
+        # has fallen off the edge of the universe.
+        for my $idx ( 0 .. $nbins - 1 ) {
+            my ( $v ) = $tree->get_range( $edge[$idx] );
+            $include[$idx] &&= $v // 0;
+        }
+    }
+
+    return __PACKAGE__->new(
+        edges   => \@edge,
+        include => \@include,
+    );
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -976,7 +1073,7 @@ sub combine_bins ( $self ) {
     my $edges   = $self->_raw_edges;
     my $include = $self->include;
 
-    my $tr = CXC::Number::Grid::Tree->new( {
+    my $tr = Tree->new( {
         'equal-p' => sub { defined $_[0] && defined $_[1] && $_[0] == $_[1] },
     } );
 
@@ -1006,7 +1103,7 @@ sub combine_bins ( $self ) {
 sub bignum ( $self ) {
     require Moo::Role;
     return Moo::Role->apply_roles_to_object(
-        __PACKAGE__->new( edges => $self->_raw_edges ),
+        __PACKAGE__->new( $self->TO_HASH ),
         __PACKAGE__ . '::Role::BigNum',
     );
 }
@@ -1026,10 +1123,11 @@ sub bignum ( $self ) {
 
 
 
+
 sub pdl ( $self ) {
     require Moo::Role;
     return Moo::Role->apply_roles_to_object(
-        __PACKAGE__->new( edges => $self->_raw_edges ),
+        __PACKAGE__->new( $self->TO_HASH ),
         __PACKAGE__ . '::Role::PDL',
     );
 }
@@ -1051,6 +1149,27 @@ sub _modify_hashr ( $self, $hash ) {
 }
 
 
+
+
+
+
+
+
+
+
+sub to_string ( $self ) {
+
+    my @edge = map { $_->numify } $self->_raw_edges->@*;
+    my @bin  = map { [ $edge[$_], $edge[ $_ + 1 ] ] } 0 .. @edge - 2;
+
+    if ( $self->_has_include ) {
+        push $bin[$_]->@*, $self->include->[$_] for 0 .. @bin - 1;
+    }
+
+    return join "\n", map { join ', ', $_->@* } @bin;
+}
+
+
 1;
 
 #
@@ -1067,8 +1186,8 @@ __END__
 
 =pod
 
-=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory nbins nedges oob ub
-unobscured Extrema bignum pdl
+=for :stopwords Diab Jerius Smithsonian Astrophysical Observatory nbins ndarrays nedges oob
+ub unobscured Extrema bignum pdl
 
 =head1 NAME
 
@@ -1076,7 +1195,7 @@ CXC::Number::Grid - A class representing a one dimensional numeric grid
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -1264,9 +1383,16 @@ return a copy of C<$grid> with a Boolean not of its include values.
 
    $ABC = $A->or( $B, $C, ..., ?\%options );
 
-Perform the logical or of grids and return a new grid.
+Perform the logical OR of grids and return a new grid.
 See C<< L</or_n> >> for a description of the options.
 
+=head2 and
+
+   $ABC = $A->and( $B, $C, ..., ?\%options );
+
+Perform the logical AND of grids and return a new grid.
+See C<< L</and_n> >> for a description of the options.
+
 =head2 combine_bins
 
   $combined = $grid->combine_bins
@@ -1302,13 +1428,21 @@ L<Math::BigFloat> objects for the following methods
 
   $bin_edges = $grid->pdl->bin_edges;
 
-Returns an object which returns piddles for the following methods
+Returns an object which returns ndarrays for the following methods
 
-  edges     -> piddle
-  bin_edges -> piddle
-  spacing   -> piddle
-  lb        -> piddle
-  ub        -> piddle
+  edges     -> ndarray
+  include   -> ndarray
+  bin_edges -> ndarray
+  spacing   -> ndarray
+  lb        -> ndarray
+  ub        -> ndarray
+
+=head2 to_string
+
+  $string = $grid->to_string
+
+Create a fairly readable string representation of the structure of a
+grid.
 
 =head1 OVERLOAD
 
@@ -1326,6 +1460,16 @@ The logical OR C<!> operator is overloaded via
 
 see L</or> for details.
 
+=head2 &
+
+   $AB = $A | $B
+
+The logical AND C<&> operator is overloaded via
+
+  $AB = $A->and($B);
+
+see L</or> for details.
+
 =head1 SUBROUTINES
 
 =head2 overlay_n
@@ -1528,7 +1672,7 @@ Add a new bin, and mark it as being excluded
 
   $grid = CXC::Number::Grid::or_n( $grid1, $grid2, ..., $gridN, ?\%options );
 
-Logical or of grids based upon their include values. For example, given
+Logical OR of grids based upon their include values. For example, given
 two grids:
 
   Grid A:
@@ -1549,12 +1693,39 @@ would be
      edges   => [ 0, 3, 4, 6, 9, 10, 11, 12, 16 ];
      include =>   [ 1, 1, 0, 1, 1,  1,  1,  0 ];
 
+The L</oob> option for the returned grid is set to the default value.
+
+=head2 and_n
+
+  $grid = CXC::Number::Grid::and_n( $grid1, $grid2, ..., $gridN, ?\%options );
+
+Logical AND of grids based upon their include values. For example, given
+two grids:
+
+  Grid A:
+
+    edges   => [ 0, 2, 4, 8, 10, 16, 18 ]
+    include => [  0, 1, 0, 1, 0,  1 ]
+
+  Grid B;
+    edges   => [ 1, 3, 6, 9, 10, 11, 16 ]
+    include => [   1, 0, 1, 0,  1,  0 ]
+
+The result of
+
+     $A & $B
+
+would be
+
+    edges => [ 0, 1, 2, 3, 4, 6, 8, 9, 10, 11, 16, 18 ];
+    include => [ 0, 0, 1, 0, 0, 0, 1, 0,  0,  0,  0 ];
+
+The L</oob> option for the returned grid is set to the default value.
+
 =head1 INTERNALS
 
 =head2 Methods
 
-=head3 _overload_not
-
 =head3 _modify_hashr
 
 This is called by MooX::Tag::TO_HASH to modify the generated hash
