@@ -10,7 +10,7 @@ use Moo::Role;
 use experimental 'signatures';
 use namespace::clean;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 sub _convert ( $self, $bignum ) {
     require Ref::Util;
@@ -20,6 +20,15 @@ sub _convert ( $self, $bignum ) {
       : $bignum->numify;
 }
 
+
+
+
+
+
+sub include ( $self ) {
+    return PDL->pdl( $self->_include );
+}
+
 #
 # This file is part of CXC-Number
 #
@@ -44,7 +53,7 @@ CXC::Number::Grid::Role::PDL - Role to return PDL objects
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -58,6 +67,8 @@ L<Math::BigFloat> arrays as piddles and passed L<Math::BigFloat> scalars as Perl
 
 =head1 INTERNALS
 
+=for Pod::Coverage include
+
 =head1 SUPPORT
 
 =head2 Bugs
